"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.shade = exports.tint = exports.argb = exports.contrast = exports.greyscale = exports.mix = exports.spin = exports.fade = exports.fadeout = exports.fadein = exports.darken = exports.lighten = exports.desaturate = exports.saturate = exports.luminance = exports.luma = exports.alpha = exports.blue = exports.green = exports.red = exports.hsvvalue = exports.hsvsaturation = exports.hsvhue = exports.lightness = exports.saturation = exports.hue = exports.hsva = exports.hsv = exports.hsla = exports.hsl = exports.rgba = exports.rgb = undefined;

var _VariableNode = require("../tree/VariableNode");

var _VariableNode2 = _interopRequireDefault(_VariableNode);

var _convert = require("../tree/convert");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var parse = function parse(color) {
    return new _VariableNode2.default("require(\"tinycolor2\")(" + (0, _convert.convertNode)(color) + ").toHex8String()");
};

var rgb = exports.rgb = function rgb(r, g, b) {
    return rgba(r, g, b, 1);
};
var rgba = exports.rgba = function rgba(r, g, b, a) {
    if (g) {
        return new _VariableNode2.default("require(\"tinycolor2\")({ r: " + (0, _convert.convertNode)(r) + ", g: " + (0, _convert.convertNode)(g) + ", b: " + (0, _convert.convertNode)(b) + ", a: " + (0, _convert.convertNode)(a) + " }).toHex8String()");
    } else {
        return parse(r);
    }
};
var hsl = exports.hsl = function hsl(h, s, l) {
    return hsla(h, s, l, 1);
};
var hsla = exports.hsla = function hsla(h, s, l, a) {
    if (s) {
        return new _VariableNode2.default("require(\"tinycolor2\")({ h: " + (0, _convert.convertNode)(h) + ", s: " + (0, _convert.convertNode)(s) + ", l: " + (0, _convert.convertNode)(l) + ", a: " + (0, _convert.convertNode)(a) + " }).toHex8String()");
    } else {
        return parse(h);
    }
};
var hsv = exports.hsv = function hsv(h, s, v) {
    return hsva(h, s, v, 1);
};
var hsva = exports.hsva = function hsva(h, s, v, a) {
    if (s) {
        return new _VariableNode2.default("require(\"tinycolor2\")({ h: " + (0, _convert.convertNode)(h) + ", s: " + (0, _convert.convertNode)(s) + ", v: " + (0, _convert.convertNode)(v) + ", a: " + (0, _convert.convertNode)(a) + " }).toHex8String()");
    } else {
        return parse(h);
    }
};

var hue = exports.hue = function hue(color) {
    return new _VariableNode2.default("Math.round(require(\"tinycolor2\")(" + (0, _convert.convertNode)(color) + ").toHsl().h)");
};
var saturation = exports.saturation = function saturation(color) {
    return new _VariableNode2.default("Math.round(require(\"tinycolor2\")(" + (0, _convert.convertNode)(color) + ").toHsl().s * 100) + \"%\"");
};
var lightness = exports.lightness = function lightness(color) {
    return new _VariableNode2.default("Math.round(require(\"tinycolor2\")(" + (0, _convert.convertNode)(color) + ").toHsl().l * 100) + \"%\"");
};

var hsvhue = exports.hsvhue = function hsvhue(color) {
    return new _VariableNode2.default("Math.round(require(\"tinycolor2\")(" + (0, _convert.convertNode)(color) + ").toHsv().h)");
};
var hsvsaturation = exports.hsvsaturation = function hsvsaturation(color) {
    return new _VariableNode2.default("Math.round(require(\"tinycolor2\")(" + (0, _convert.convertNode)(color) + ").toHsv().s * 100) + \"%\"");
};
var hsvvalue = exports.hsvvalue = function hsvvalue(color) {
    return new _VariableNode2.default("Math.round(require(\"tinycolor2\")(" + (0, _convert.convertNode)(color) + ").toHsv().v * 100) + \"%\"");
};

var red = exports.red = function red(color) {
    return new _VariableNode2.default("require(\"tinycolor2\")(" + (0, _convert.convertNode)(color) + ").toRgb().r");
};
var green = exports.green = function green(color) {
    return new _VariableNode2.default("require(\"tinycolor2\")(" + (0, _convert.convertNode)(color) + ").toRgb().g");
};
var blue = exports.blue = function blue(color) {
    return new _VariableNode2.default("require(\"tinycolor2\")(" + (0, _convert.convertNode)(color) + ").toRgb().b");
};
var alpha = exports.alpha = function alpha(color) {
    return new _VariableNode2.default("Math.round(require(\"tinycolor2\")(" + (0, _convert.convertNode)(color) + ").toRgb().a * 100) / 100");
};

var luma = exports.luma = function luma(color) {
    return new _VariableNode2.default("require(\"tinycolor2\")(" + (0, _convert.convertNode)(color) + ").getLuminance() * 100");
};
var luminance = exports.luminance = function luminance(color) {
    var compute = function compute(rgb) {
        return (0.2126 * rgb.r / 255 + 0.7152 * rgb.g / 255 + 0.0722 * rgb.b / 255) * rgb.a;
    };
    return new _VariableNode2.default("(" + compute + ")(require(\"tinycolor2\")(" + (0, _convert.convertNode)(color) + ").toRgb()) * 100");
};

var saturate = exports.saturate = function saturate(color, amount, method) {
    return new _VariableNode2.default("require('tinycolor2')(" + (0, _convert.convertNode)(color) + ").saturate(parseFloat(" + (0, _convert.convertNode)(amount) + ")).toHex8String()");
};
var desaturate = exports.desaturate = function desaturate(color, amount, method) {
    return new _VariableNode2.default("require('tinycolor2')(" + (0, _convert.convertNode)(color) + ").desaturate(parseFloat(" + (0, _convert.convertNode)(amount) + ")).toHex8String()");
};

var lighten = exports.lighten = function lighten(color, amount, method) {
    return new _VariableNode2.default("require('tinycolor2')(" + (0, _convert.convertNode)(color) + ").lighten(parseFloat(" + (0, _convert.convertNode)(amount) + ")).toHex8String()");
};
var darken = exports.darken = function darken(color, amount, method) {
    return new _VariableNode2.default("require('tinycolor2')(" + (0, _convert.convertNode)(color) + ").darken(parseFloat(" + (0, _convert.convertNode)(amount) + ")).toHex8String()");
};

var fadein = exports.fadein = function fadein(color, amount, method) {
    var compute = function compute(color, amount, method) {
        var alpha = void 0;
        if (typeof method !== 'undefined' && method === 'relative') {
            alpha = color.getAlpha() + color.getAlpha() * amount / 100;
        } else {
            alpha = color.getAlpha() + amount / 100;
        }
        color.setAlpha(Math.min(1, Math.max(0, alpha)));
        return color.toRgbString();
    };
    return new _VariableNode2.default("(" + compute + ")(require(\"tinycolor2\")(" + (0, _convert.convertNode)(color) + "), parseFloat(" + (0, _convert.convertNode)(amount) + "), " + (0, _convert.convertNode)(method) + ")");
};
var fadeout = exports.fadeout = function fadeout(color, amount, method) {
    var compute = function compute(color, amount, method) {
        var alpha = void 0;
        if (typeof method !== 'undefined' && method === 'relative') {
            alpha = color.getAlpha() - color.getAlpha() * amount / 100;
        } else {
            alpha = color.getAlpha() - amount / 100;
        }
        color.setAlpha(Math.min(1, Math.max(0, alpha)));
        return color.toRgbString();
    };
    return new _VariableNode2.default("(" + compute + ")(require(\"tinycolor2\")(" + (0, _convert.convertNode)(color) + "), parseFloat(" + (0, _convert.convertNode)(amount) + "), " + (0, _convert.convertNode)(method) + ")");
};
var fade = exports.fade = function fade(color, amount) {
    return new _VariableNode2.default("require(\"tinycolor2\")(" + (0, _convert.convertNode)(color) + ").setAlpha(parseFloat(" + (0, _convert.convertNode)(amount) + ") / 100).toRgbString()");
};
var spin = exports.spin = function spin(color, amount) {
    return new _VariableNode2.default("require(\"tinycolor2\")(" + (0, _convert.convertNode)(color) + ").spin(parseFloat(" + (0, _convert.convertNode)(amount) + ")).toHex8String()");
};
var mix = exports.mix = function mix(color1, color2, weight) {
    return new _VariableNode2.default("require(\"tinycolor2\").mix(" + (0, _convert.convertNode)(color1) + ", " + (0, _convert.convertNode)(color2) + ", parseFloat(" + (0, _convert.convertNode)(weight) + ")).toHex8String()");
};
var greyscale = exports.greyscale = function greyscale(color) {
    return desaturate(color, 100);
};

var contrast = exports.contrast = function contrast(color, dark, light, threshold) {
    var compute = function compute(t, color, dark, light, threshold) {
        if (typeof light === 'undefined') {
            light = "#FFF";
        }
        if (typeof dark === 'undefined') {
            dark = "#000";
        }
        // Figure out which is actually light and dark:
        if (t(dark).getLuminance() > t(light).getLuminance()) {
            var _t = light;
            light = dark;
            dark = _t;
        }
        if (typeof threshold === 'undefined') {
            threshold = 0.43;
        } else {
            threshold = parseFloat(threshold) / 100;
        }
        if (t(color).getLuminance() < threshold) {
            return light;
        } else {
            return dark;
        }
    };
    return new _VariableNode2.default("(" + compute + ")(require(\"tinycolor2\"), " + (0, _convert.convertNode)(color) + ", " + (0, _convert.convertNode)(dark) + ", " + (0, _convert.convertNode)(light) + ", " + (0, _convert.convertNode)(threshold) + ")");
};

var argb = exports.argb = function argb(color) {
    return new _VariableNode2.default((0, _convert.convertNode)(rgba(color)) + ".replace(/#(\\w{6})(\\w{2})/, \"#$2$1\")");
};
// color
var tint = exports.tint = function tint(color, weight) {
    return mix("#ffffff", color, weight);
};
var shade = exports.shade = function shade(color, weight) {
    return mix("#000000", color, weight);
};