"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.round = exports.acos = exports.asin = exports.atan = exports.cos = exports.sin = exports.tan = exports.abs = exports.sqrt = exports.floor = exports.ceil = undefined;

var _VariableNode = require("../tree/VariableNode");

var _VariableNode2 = _interopRequireDefault(_VariableNode);

var _convert = require("../tree/convert");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var apply = function apply(fn, n) {
    return fn(parseFloat(n)) + ('' + n).replace(/[\d.-]*/, "");
};

var applyAngle = function applyAngle(fn, n) {
    var funit = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : "";

    var angle = {
        rad: 1,
        deg: Math.PI / 180,
        grad: Math.PI / 200,
        turn: Math.PI * 2
    };
    var unit = ('' + n).replace(/[\d.-]*/, "") || "rad";
    return fn(parseFloat(n) * angle[unit]) + funit;
};

var ceil = exports.ceil = function ceil(value) {
    return new _VariableNode2.default("(" + apply + ")(Math.ceil, " + (0, _convert.convertNode)(value) + ")");
};
var floor = exports.floor = function floor(value) {
    return new _VariableNode2.default("(" + apply + ")(Math.floor, " + (0, _convert.convertNode)(value) + ")");
};
var sqrt = exports.sqrt = function sqrt(value) {
    return new _VariableNode2.default("(" + apply + ")(Math.sqrt, " + (0, _convert.convertNode)(value) + ")");
};
var abs = exports.abs = function abs(value) {
    return new _VariableNode2.default("(" + apply + ")(Math.abs, " + (0, _convert.convertNode)(value) + ")");
};
var tan = exports.tan = function tan(value) {
    return new _VariableNode2.default("(" + applyAngle + ")(Math.tan, " + (0, _convert.convertNode)(value) + ")");
};
var sin = exports.sin = function sin(value) {
    return new _VariableNode2.default("(" + applyAngle + ")(Math.sin, " + (0, _convert.convertNode)(value) + ")");
};
var cos = exports.cos = function cos(value) {
    return new _VariableNode2.default("(" + applyAngle + ")(Math.cos, " + (0, _convert.convertNode)(value) + ")");
};
var atan = exports.atan = function atan(value) {
    return new _VariableNode2.default("(" + applyAngle + ")(Math.atan, " + (0, _convert.convertNode)(value) + ", \"rad\")");
};
var asin = exports.asin = function asin(value) {
    return new _VariableNode2.default("(" + applyAngle + ")(Math.asin, " + (0, _convert.convertNode)(value) + ", \"rad\")");
};
var acos = exports.acos = function acos(value) {
    return new _VariableNode2.default("(" + applyAngle + ")(Math.acos, " + (0, _convert.convertNode)(value) + ", \"rad\")");
};
var round = exports.round = function round(value) {
    var fraction = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    return new _VariableNode2.default("(" + apply + ")(num => num.toFixed(" + fraction.value + "), " + (0, _convert.convertNode)(value) + ")");
};